
import java.util.*;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;

class expr_assign {

    public static void main(String[] args) {
        BufferedReader br = null;
        try {
           br = new BufferedReader(new InputStreamReader(System.in));
        } catch (Exception e) {
           System.err.println("problem with BufferedReader");
           System.exit(1);
        }
        String theLine;
        Map<String,Long> store = new HashMap<String, Long>();
        while(true) {
                try {
                    theLine = br.readLine();
                } catch (IOException ioe) {
                    break;
                }
        if (theLine == null) break;
        if (theLine.equals("quit")) break;
        if (theLine.equals("")) continue;
        Tokenizer t = new Tokenizer(false, theLine);
        String token;
        token = t.token();
        if (token == null) break;
        //System.out.println("just before evaluator");
        Evaluator2 e = new Evaluator2(token, t, store);
        //t.setDebug(true);
        Long theResult = (long) 0;
        try {
            theResult = e.stmtEval(0);
            e.accept(null);
            if (theResult != null) System.out.println(theResult);
        } catch (SyntaxException2 se) {
            System.out.println("syntax error: " + se.getMessage());
        } catch (IllegalArgumentException iae) {
            System.out.println("evaluation error: division by zero");
        }
    }
    }
}

class SyntaxException2 extends Exception  {
    public SyntaxException2()  { super(); }
    public SyntaxException2(String message)  {super(message); }
    public SyntaxException2(String message, Exception inner) { super(message, inner); }

    // A constructor is needed for serialization when an 
    // exception propagates from a remoting server to the client.  
    //protected InvalidDepartmentException(System.Runtime.Serialization.SerializationInfo info,
    //   System.Runtime.Serialization.StreamingContext context) { }
}


class Evaluator2 {

    private Tokenizer t;
    private String theToken;
    private Map<String,Long> theStore;
    private boolean TRACING = false;
    
    public Evaluator2(String token, Tokenizer Tok) {
    t = Tok;
    theToken = token;
        theStore = null;
    }

    public Evaluator2(String token, Tokenizer Tok, Map<String,Long> store) {
        t = Tok;
        theToken = token;
        theStore = store;
    }

    private void dump() {
        for (String s : theStore.keySet()) {
            System.out.printf("%s: %d%n", s, theStore.get(s));
        }
    }

    private void trace(int depth, String fname) {
        if (TRACING) {
            for (int i=0; i<depth; i++) System.out.print("    ");
            System.out.printf("%s: token is %s%n", fname, theToken);
        }
    }
    /* Grammar:
    stmt   ::= expr | 'set' ident '=' expr      // new
    expr   ::= term { addop term }
    term   ::= factor { mulop factor }
    factor ::= number | '(' expr ')' | identifier   // changed
    /* */


    public Long stmtEval(int d) throws SyntaxException2 {
        trace(d,"stmtEval");
        if (t.equals(theToken, "dump")) {
            accept("dump");
            dump();
            return null;
        } else if (t.equals(theToken, "trace")) {
            accept("trace");
            TRACING = ! TRACING;    // doesn't work; sets it only for this Evaluator
            System.out.println("TRACING set to " + TRACING);
            return null;
        } else if (t.equals(theToken,"set")) {
            accept("set");
            String identname = theToken;
            if (!t.isIdent(identname)) {
            throw new SyntaxException2 ("expected <identifier>; got \""+ theToken+ "\"");
            }
            theToken = t.token();
            accept("=");
            long result = exprEval(d+1);
            theStore.put(identname, result);
            return null;
        } else {
            long result = exprEval(d+1);
            return result;
        }
    }

    public long exprEval(int d) throws SyntaxException2 {
        trace(d,"exprEval");
        long result = termEval(d+1);
        while (t.equals(theToken, "+") || t.equals(theToken, "-")) {
            String addop = theToken;
            theToken = t.token();
            long res2 = termEval(d+1);
            if (addop.equals("+")) result += res2;
            if (addop.equals("-")) result -= res2;
        }
        return result;
    }

    private long termEval(int d)  throws SyntaxException2 {
        trace(d,"termEval");
        long result = factorEval(d+1);
        while (t.equals(theToken, "*") || t.equals(theToken, "/") || t.equals(theToken, "%")) {
            String mulop = theToken;
            theToken = t.token();
            long res2 = factorEval(d+1);
            if (mulop.equals("*")) result *= res2;
            if (mulop.equals("/")) {
                if (res2 == 0) throw new IllegalArgumentException("division by zero");
                result /= res2;
            }
            if (mulop.equals("%")) result %= res2;
        }
        return result;
    }

    private long factorEval(int d) throws SyntaxException2 {
        trace(d, "factorEval");
        if (t.isNumber(theToken)) {
            long result = new Long(theToken);
            theToken = t.token();
            return result;
        } else if (theToken.equals("(")) {
            accept("(");
            long result = exprEval(d+1);
            accept(")");
            return result;
            } else if (t.isIdent(theToken)) {           // new
            if (!theStore.containsKey(theToken)) {
                throw new SyntaxException2("unassigned identifier: \"" + theToken + "\"");
            }
            long result = theStore.get(theToken);
            theToken = t.token();
            return result;
        } else {
            throw new SyntaxException2 ("illegal factor token: \"" + theToken + "\"");
        }
    }

    // must be in this class, *not* Tokenizer
    public void accept(String expected) throws SyntaxException2 {
        if (theToken == null && expected == null 
            || theToken !=null && theToken.equals(expected)) {
                theToken = t.token();
        } else {
        if (theToken == null) theToken = "<EOL>";
        throw new SyntaxException2 ("expected \""+expected+"\"; got \""+ theToken+ "\"");
            //t.error("expected \""+expected+"\"; got \""+ theToken+ "\"");
        }
    }
}
